import javax.swing.*;
import javax.swing.plaf.basic.BasicArrowButton;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Random;

import Dop.DopClassRect;
import Dop.DopClassTriangle;
import Dop.NumberOfWheels;
import Entities.*;
import DrawingObjects.*;
import MovementStrategy.*;

public class GameFrame extends JDialog {
    protected DrawingCar SelectedCar;

    GameFrame() {
        this.setSize(710, 535);
        this.setTitle("DumpTruck");
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.setModalExclusionType(ModalExclusionType.APPLICATION_EXCLUDE);
        this.setModalityType(ModalityType.APPLICATION_MODAL);
        this.setResizable(false);
        this.setLocationRelativeTo(null);

        GamePanel panel = new GamePanel();
        this.add(panel);

        this.setVisible(true);
    }
    public DrawingCar getSelectedCar(){
        return SelectedCar;
    }

    public class GamePanel extends JPanel {
        static final int SCREEN_W = 700;
        static final int SCREEN_H = 500;
        int xx = 0;
        int yy = 0;
        private DrawingCar _drawningCar;
        private AbstractStrategy _abstractStrategy;
        JColorChooser dialogColor = new JColorChooser();


        GamePanel() {
            this.setLayout(null);
            this.setPreferredSize(new Dimension(SCREEN_W, SCREEN_H));
            GridBagConstraints layers = new GridBagConstraints();

            JButton buttonCar = new JButton("Создать грузовик");
            buttonCar.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    Random random = new Random();
                    Color color = Color.BLACK;

                    Color choosenColor = JColorChooser.showDialog(getParent(), "Выберите цвет", Color.WHITE);
                    if(choosenColor != null){
                        color = choosenColor;
                    }

                    _drawningCar = new DrawingCar(random.nextInt(100, 300),
                            random.nextInt(1000, 3000),
                            color,
                            GamePanel.SCREEN_W, GamePanel.SCREEN_H);
                    _drawningCar.SetPosition(random.nextInt(10, 100), random.nextInt(10,
                            100));

                    int ornament = random.nextInt(1, 4);
                    switch (ornament){
                        case 1:
                            _drawningCar.wheels = new NumberOfWheels(random.nextInt(2, 5));
                            break;
                        case 2:
                            _drawningCar.wheels = new DopClassRect(random.nextInt(2, 5));
                            break;
                        default:
                            _drawningCar.wheels = new DopClassTriangle(random.nextInt(2, 5));
                            break;
                    }
                    repaint();
                }
            });
            buttonCar.setBounds(20 + 150, 390, 120, 30);
            this.add(buttonCar);

            JButton buttonDumpCar = new JButton("Создать самосвал");
            buttonDumpCar.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    Random random = new Random();
                    Color color = Color.BLACK; Color colorAdd = Color.WHITE;

                    Color choosenColor = JColorChooser.showDialog(getParent(), "Выберите цвет", Color.WHITE);
                    Color choosenAddColor = JColorChooser.showDialog(getParent(), "Выберите цвет", Color.WHITE);

                    if(choosenColor != null){
                        color = choosenColor;
                    }
                    if(choosenAddColor != null){
                        colorAdd = choosenAddColor;
                    }

                    _drawningCar = new DrawingDumpTruck(random.nextInt(100, 300),
                            random.nextInt(1000, 3000),
                            color, colorAdd, random.nextBoolean(),random.nextBoolean(),random.nextBoolean(),
                            GamePanel.SCREEN_W, GamePanel.SCREEN_H); //TODO
                    _drawningCar.SetPosition(random.nextInt(10, 100), random.nextInt(10,
                            100));

                    int ornament = random.nextInt(1, 4);
                    switch (ornament){
                        case 1:
                            _drawningCar.wheels = new NumberOfWheels(random.nextInt(2, 5));
                            break;
                        case 2:
                            _drawningCar.wheels = new DopClassRect(random.nextInt(2, 5));
                            break;
                        default:
                            _drawningCar.wheels = new DopClassTriangle(random.nextInt(2, 5));
                            break;
                    }


                    repaint();
                }
            });
            buttonDumpCar.setBounds(  20, 390, 120, 30);
            this.add(buttonDumpCar);

            JTextField textStrategy = new JTextField();
            textStrategy.setBounds(550, 20, 120, 30);
            this.add(textStrategy);

            JButton buttonStep = new JButton("Шаг");
            buttonStep.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    if (_drawningCar == null)
                    {
                        return;
                    }
                    if (textStrategy.getText() != null)
                    {
                        switch(textStrategy.getText())
                        {
                            case "center":

                                _abstractStrategy = new MoveToCenter();
                                break;
                            case "border":
                                _abstractStrategy = new MoveToBorder();
                                break;
                        };

                        if (_abstractStrategy == null)
                        {

                            return;
                        }
                        _abstractStrategy.SetData((IMoveableObject) _drawningCar, SCREEN_W, SCREEN_H);
                        //textStrategy.setText();
                    }
                    if (_abstractStrategy == null)
                    {

                        return;
                    }
                    System.out.println("step");
                    _abstractStrategy.MakeStep();
                    repaint();
                    if (_abstractStrategy.GetStatus() == Status.Finish)
                    {
                        //comboBoxStrategy.Enabled = true;
                        _abstractStrategy = null;
                    }
                }
            });
            buttonStep.setBounds(550, 60, 70, 30);
            this.add(buttonStep);

            JButton up = new BasicArrowButton(BasicArrowButton.NORTH);
            up.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    _drawningCar.MoveTransport(Direction.Up);
                    repaint();
                }
            });
            JButton left = new BasicArrowButton(BasicArrowButton.WEST);
            left.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    _drawningCar.MoveTransport(Direction.Left);
                    repaint();
                }
            });
            JButton down = new BasicArrowButton(BasicArrowButton.SOUTH);
            down.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    _drawningCar.MoveTransport(Direction.Down);
                    repaint();
                }
            });
            JButton right = new BasicArrowButton(BasicArrowButton.EAST);
            right.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    _drawningCar.MoveTransport(Direction.Right);
                    repaint();
                }
            });


            up.setBounds(570, 350, 30, 30);
            this.add(up);

            down.setBounds(570, 350 + 40, 30, 30);
            this.add(down);

            left.setBounds(570 - 40, 350 + 40, 30, 30);
            this.add(left);

            right.setBounds(570 + 40, 350 + 40, 30, 30);
            this.add(right);

            JButton buttonSelectedCar = new JButton("Выбрать машину");

            buttonSelectedCar.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    SelectedCar = _drawningCar;
                    dispose();

                }
            });
            buttonSelectedCar.setBounds(510, 100, 150, 30);
            this.add(buttonSelectedCar);
        }
        private void Draw()
        {
            if (_drawningCar == null)
            {
                return;
            }

            Graphics gr =new DebugGraphics();
            _drawningCar.DrawTransport(gr);

        }
        @Override
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            draw(g);
        }

        public void draw(Graphics g) {
            if (_drawningCar != null) {
                _drawningCar.DrawTransport(g);
            }
        }

    }
    public Color ChooseColor(JFrame MonorailFrame){
        JColorChooser dialog = new JColorChooser();
        Color res = JColorChooser.showDialog(MonorailFrame, "Выберите цвет", Color.WHITE);
        return res;
    }
}


